﻿-- Author      : RisM
-- Create Date : 6/28/2009 3:56:24 PM

SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

-------------------------------------------------------------------------------
-- DATA STRUCTURES (? not sure what to call this category)
-------------------------------------------------------------------------------


function SpeakinSpell:DeleteSpell(key)
	-- delete the current selection from the SavedData and RuntimeData tables
	SpeakinSpellSavedData.EventTable[key] = nil
	
	if not self.RuntimeData then
		-- we're still initializing, so the rest of this stuff doesn't matter
		return
	end
	
	-- remove runtime data for messaging
	self.RuntimeData.LastMessage[key] = nil
	
	-- update GUI controls
	self.OptionsGUI.args.CurrentMessagesGUI.args.EventSelect.values[key] = nil
	
	-- update selected item
	self.OptionsGUIStates.MessageSettings.SelectedEventKey = nil
	
	-- restore the spell to the list of new detected
	-- so the user doesn't have to retrigger it
	-- self:RecordNewSpell(EventTableEntry.DetectedEvent)
	-- NO! could be outdated and bugged event
	-- just make the user recreate it	
	
	self:CurrentMessagesGUI_ValidateSelectedEvent() -- make sure we still have a valid spell selection
	self:CurrentMessagesGUI_Refresh()
end


function SpeakinSpell:MatchesFilter(DetectedEvent, SelectedEventTypeFilter)
	if not SelectedEventTypeFilter then
		return true -- everything matches if no filter is selected
	end
	
	return (SelectedEventTypeFilter == "ALL" or SelectedEventTypeFilter == DetectedEvent.type)
end


-- RecordNewEvent creates and maintains a list of new spells that don't have messages associated with them
-- this list is used to present the user with a list of options when she wants to start announcing a new spell
function SpeakinSpell:RecordNewEvent(DetectedEvent)
	local funcname = "RecordNewEvent"
	
	-- if we already recorded this spell, don't add a duplicate, unless this is a different rank
	-- NOTE: RecordNewEvent is called for both ranked, and rankless versions of the event
	if not SpeakinSpellSavedData.NewEventsDetected[DetectedEvent.key] then
		self:DebugMsg(funcname, DetectedEvent.SpellLink)
		SpeakinSpellSavedData.NewEventsDetected[DetectedEvent.key] = DetectedEvent
		-- update the options GUI if it exists
		if self.OptionsGUI then
			-- if this is a new macro, then we're going to want to open the Create New interface to show it
			-- if that is the case, we need to make sure macros can be shown by the selected filter
			if DetectedEvent.type == "MACRO" then
				if not self:MatchesFilter(DetectedEvent, self.OptionsGUIStates.CreateNew.SelectedEventTypeFilter) then
					self.OptionsGUIStates.CreateNew.SelectedEventTypeFilter = "MACRO"
				end
			end
			
			if self:MatchesFilter(DetectedEvent, self.OptionsGUIStates.CreateNew.SelectedEventTypeFilter) then
				-- add this to the list
				self.OptionsGUI.args.CreateNew.args.NewSpellNameSelect.values[DetectedEvent.key] = DetectedEvent.DisplayName
				-- re sort the list
				table.sort(self.OptionsGUI.args.CreateNew.args.NewSpellNameSelect.values)
				-- make this new item the selection
				self.OptionsGUIStates.CreateNew.SelectedEventKey = DetectedEvent.key
				-- if this is a new macro, open the Create New interface
				if DetectedEvent.type == "MACRO" then
					self:ShowCreateNew()
				end
			end
		end
	end
end


function SpeakinSpell:FormatDisplayName(de)
	-- create a display name for this event
	if de.rank ~= "" then
		return L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.name.." ("..de.rank..")"
	else
		if de.type == "UNIT_SPELLCAST_SENT" then
			return L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.name.." (Any Rank)" --TODOLATER: localize
		else --other supported events (buffs, debuffs, macros, and system events) donever 't have ranks
			return L.EVENTTYPES.IN_SPELL_LIST[de.type]..de.name
		end
	end
end


function SpeakinSpell:SafeGetSpellLink(de)
	local SpellLink,_ = GetSpellLink(de.name, de.rank)
	if SpellLink then
		return SpellLink
	else
		return "["..de.name.."]"
	end
end



-- returns rankedkey, ranklesskey
-- if not a rankable event, then ranklesskey will be nil
-- TODOFUTURE: this could return an ordered list of potential keys?
-- maybe for linking UNIT_SPELLCAST_SENT to the matching SPELL_AURA_GAIN 
-- and/or for making speech sets that apply to many spells with the same speech settings
function SpeakinSpell:MakeEventTableKeys( de )
	-- create a unique key for this event
	local rankedkey		= string.gsub( string.upper(de.type..de.name..de.rank), " ", "_" ) -- ACTIVE 3.1.2.07 format
	local ranklesskey	= string.gsub( string.upper(de.type..de.name),			" ", "_" ) -- ACTIVE 3.1.2.07 format
	
	if (de.type == "UNIT_SPELLCAST_SENT") and (rankedkey ~= ranklesskey) then
		return rankedkey, ranklesskey
	else
		return rankedkey, nil
	end
end


function SpeakinSpell:CreateDetectedEvent( DetectedEventStub )
	local funcname = "CreateDetectedEvent"
	
	local de = self:CopyTable(DetectedEventStub)
	
	-- create a spell link, if possible
	-- the stub will set SpellLink = true if the event is a spell
	-- else will we make a fake one
	de.SpellLink = self:SafeGetSpellLink(de)
	
	-- validate the target - get a backup target from selection, focus, or assumed self-cast
	if not de.target or de.target == "" or de.target == "Unknown" then
		de.target = self:GetDefaultTarget()
	end
	
	-- fetch additional target information
	de.targetclass = UnitClass(de.target)
	de.targetrace = UnitRace(de.target)

	--setup <pet>
	de.pet = UnitName("pet")
	
	-- create a display name for this event
	de.DisplayName = self:FormatDisplayName(de)
	
	return de
end


function SpeakinSpell:CheckSpellFrequency( de )
	-- check the frequency setting against a random number to determine if we should speak or not
	local Chance = math.random(1,100)
	local Frequency = de.EventTableEntry.Frequency*100

	if Chance > Frequency then
		self:DebugMsg( "CheckSpellFrequency", string.format("random chance failed for '%s' rolled %d > %d", de.SpellLink, Chance, Frequency) )
		return false
	end
	
	return true
end


function SpeakinSpell:CheckSpellCooldown( de )
	local funcname = "CheckSpellCooldown"
	
	-- check for a cooldown limit on announcing this spell

	-- NOTE: if we have not spoken for this spell yet, then
	--		self.RuntimeData.LastMessageTime or self.RuntimeData.LastMessageTime[spellname] might not exist
	--		in that case, set elapsed = cooldown, so that it will seem like enough time has passed
	local now = GetTime()
	if self.RuntimeData.LastMessageTime == nil then
		self.RuntimeData.LastMessageTime = {}
	end
	local last = self.RuntimeData.LastMessageTime[de.key]
	if last == nil then
		last = 0
	end
	local elapsed = de.EventTableEntry.Cooldown
	if last then
		elapsed = now - last
	end
	-- Now finally actually check if the cooldown is in effect
	if elapsed < de.EventTableEntry.Cooldown then
		-- the cooldown is in effect, so be silent
		self:DebugMsg( funcname, string.format("messaging on cooldown: %s: now:%d, last:%d, elapsed:%d, CD:%d", de.SpellLink, now, last, elapsed, de.EventTableEntry.Cooldown) )
		return false
	end
	
	return true
end


function SpeakinSpell:AllowSpeakForSpell( de )
	local funcname = "AllowSpeakForSpell"
	
	if de.EventTableEntry.DisableAnnouncements then
		self:DebugMsg(funcname,"OnEventDetected - messages for " .. de.SpellLink .. " are disabled")
		return false
	end
	
	if not SpeakinSpellSavedData.EnableAllMessages then
		self:DebugMsg(funcname,"OnEventDetected - all SpeakinSpell messages are disabled")
		return false
	end
	
	if not self:CheckSpellFrequency( de ) then
		return false
	end
	
	if not self:CheckSpellCooldown( de ) then
		return false
	end
	
	return true
end

